#!/usr/bin/perl -w
###############################################################################
##    Copyright (C) 2002 by Eric Gerbier
##    Bug reports to: gerbier@users.sourceforge.net
##    $Id: afick-tk.pl,v 1.22 2004/01/08 09:08:54 gerbier Exp $
##
##    This program is free software; you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation; either version 2 of the License, or
##    (at your option) any later version.
##
##    This program is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
################################################################################
# this program just provide a graphical interface to afick
# it just launch afick commands
################################################################################
# rem : can not work in tainted mode
################################################################################

use strict;
use Getopt::Long;    # arg analysis

use Tk;              # interface
use Tk::ROText;      # to have read-only text
use Tk::ProgressBar; # the progress bar
use Tk::Adjuster;    # a separator
use Tk::LabFrame;    # frame with a label

# global var
my $version = "0.6.0";
my $change_text;      # text widget for change display
my $warning_text;     # text widget for warnings
my $progress_text;    # text widget for progress file name
my $progress_bar;     # progress bar
my $ToggleProgress   = 1;    # checkbox var
my $progress_percent = 0;    # progress bar var

# command arguments
my $configfile = '';
my $help       = '';
my $verbose    = '';
my $Version    = '';
my $Report_full_newdel;
my $Warn_missing_file;
my $Warn_dead_symlinks;
my $Running;
my $Timing;

# labels for Optionmenu widgets (option configuration)
my @verbose_values     = qw(verbose_config verbose noverbose);
my @missing_values     = qw(missing_files_config missing_files nomissing_files);
my @dead_sym_values    = qw(dead_symlinks_config dead_symlinks nodead_symlinks);
my @full_newdel_values = qw(full_newdel_config full_newdel nofull_newdel);
my @running_files_values =
  qw(running_files_config running_files norunning_files);
my @timing_values = qw(timing_config timing notiming);

# default file
# be careful : same code in afick.pl
my $default_conf;
if ( $^O =~ m/linux/ ) {
	$default_conf = "/etc/afick.conf";
}
elsif ( $^O =~ m/^MSWin/ ) {
	$default_conf = "xp.conf";
}
else {
	$default_conf = "afick.conf";
}

#####################################################################
# used to clear output screen before each command
sub clear_text($) {
	my $widget = shift (@_);
	$widget->delete( '1.0', 'end' );
}

#####################################################################
# print warnings in warning section and console
sub warning ($) {
	my $text = shift (@_);
	$warning_text->insert( 'end', "$text\n" );
	$warning_text->update();
	warn "$text\n";
}
#####################################################################
# used to spawn afick commands
sub do_action($) {
	my $arg = shift (@_);

	$arg .= " -c \"$configfile\"" if ($configfile);

	# force values (overload values in config file)
	if ( $verbose eq $verbose_values[1] ) {
		$arg .= " --verbose";
	}
	elsif ( $verbose eq $verbose_values[2] ) {
		$arg .= " --noverbose";
	}

	if ( $Warn_missing_file eq $missing_values[1] ) {
		$arg .= " --missing_files";
	}
	elsif ( $Warn_missing_file eq $missing_values[2] ) {
		$arg .= " --nomissing_files";
	}

	if ( $Report_full_newdel eq $full_newdel_values[1] ) {
		$arg .= " --full_newdel";
	}
	elsif ( $Report_full_newdel eq $full_newdel_values[2] ) {
		$arg .= " --nofull_newdel";
	}

	if ( $Warn_dead_symlinks eq $dead_sym_values[1] ) {
		$arg .= " --dead_symlinks";
	}
	elsif ( $Warn_dead_symlinks eq $dead_sym_values[2] ) {
		$arg .= " --nodead_symlinks";
	}

	if ( $Warn_dead_symlinks eq $dead_sym_values[1] ) {
		$arg .= " --dead_symlinks";
	}
	elsif ( $Warn_dead_symlinks eq $dead_sym_values[2] ) {
		$arg .= " --nodead_symlinks";
	}

	if ( $Running eq $running_files_values[1] ) {
		$arg .= " --running_files";
	}
	elsif ( $Running eq $running_files_values[2] ) {
		$arg .= " --norunning_files";
	}

	if ( $Timing eq $timing_values[1] ) {
		$arg .= " --timing";
	}
	elsif ( $Timing eq $timing_values[2] ) {
		$arg .= " --notiming";
	}

	# progress Checkbutton
	$arg .= " --progress" if ($ToggleProgress);

	print "arg = $arg \n" if ( $verbose eq $verbose_values[1] );

	if ( !open( ACTION, "afick.pl $arg  2>&1 |" ) ) {
		warning("can not execute afick.pl : $!");
		return;
	}
	else {

		# save current cursor
		my $cursor = $change_text->cget('-cursor');

		# now a waiting cursor
		$change_text->configure( -cursor => 'clock' );
		clear_text($change_text);
		$change_text->update();
		clear_text($warning_text);
		$warning_text->update();
		clear_text($progress_text);
		$progress_text->update();

		my $refresh = 0;
		my $nb      = 0;
		my $total   = 1;    # to avoid a divide by zero
		$progress_percent = 0;
		while (<ACTION>) {
			if ( $_ =~ m/^progress total (\d+)/ ) {

				# get number of file in database
				$total = $1;
			}
			elsif ( $_ =~ m/^progress (.*)/ ) {

				# get current scanned file
				$nb++;
				$progress_percent = ( $nb * 100 ) / $total;
				$progress_bar->update();

				my $fic = $1;
				clear_text($progress_text);
				$progress_text->insert( 'end', $fic );
				$progress_text->update();
			}
			elsif ( $_ =~ m/^WARNING: (.*)/ ) {
				warning($1);
			}
			else {
				$change_text->insert( 'end', $_ );

				# rafraichissement regulier
				$refresh++;
				if ( $refresh == 10 ) {
					$refresh = 0;
					$change_text->update();
				}
			}
		}    # while
		close(ACTION);

		# go to end of both screens
		$change_text->see('end');
		$warning_text->see('end');

		# restore cursor
		$change_text->configure( -cursor => $cursor );
		clear_text($progress_text);
	}
}

#####################################################################
# display help page
sub do_help($) {
	my $main = shift;

	my $top = $main->Toplevel;
	$top->Label( -text => "help" )->pack();
	$top->Button( -text => "quit", -command => [ $top => 'destroy' ] )->pack();
	my $text = $top->Scrolled(
		'ROText',
		-scrollbars => 'e',
		-height     => 25,
		-width      => 128,
		-wrap       => 'word'
	)->pack( -side => "left", -expand => 1, -fill => 'both' );

	my $baratin =
	  "this is a graphical interface to afick (another file integrity checker)
to monitor file system changes

actions buttons :
-----------------
init : to create the database
update : compare and update the database
compare : compare the files with the database
print : print database content
save  : save output screen to a local file
load  : display a saved outpout in outpout screen
help : this page
about : legal informations
exit : to quit this interface

the change section :
--------------------
display file changes

the warning section :
---------------------
display afick errors and warnings

the progress section :
----------------------
can be activated/desactivated by the 'display progress' checkbutton
it is useful fo follow afick progress (compare on update mode only)
it display the currently scanned file
then a progress bar, from 0 to 100%, with a line each 10%

configuration section :
-----------------------
a first line to work on configuration file :
- configuration file : to enter a configuration file name
- select... : to select a configuration file with a fileselector widget
- view/edit : to view or edit the selected configuration file

then a line of radio-buttons for option, corresponding to command-line options :
- verbose : produce more verbose output (for debugging purpose)
- missing_files : warn about files in config file but missing
- dead_links : warn about dead symlinks
- full_newdel : produce a full report on new/deleted directories
- running_files : warn about files modified during program run
- timing : print timing statistics

each offers 3 choices :
- keep config file choice (*_config)
- force parameter
- force opposite parameter (no*)
";

	$text->insert( 'end', $baratin );
	$text->see('1.0');
}
#####################################################################
# display about page
sub do_about($) {
	my $main = shift;

	my $top = $main->Toplevel;
	$top->Label( -text => "about" )->pack();
	$top->Button( -text => "quit", -command => [ $top => 'destroy' ] )->pack();
	my $text = $top->Scrolled(
		'ROText',
		-scrollbars => 'e',
		-height     => 25,
		-width      => 128,
		-wrap       => 'word'
	)->pack( -side => "left", -expand => 1, -fill => 'both' );

	my $baratin = "afick-tk version $version : a graphical interface to afick\n
url  : http://afick.sourceforge.net
Copyright (c) 2002 Eric Gerbier <gerbier\@users.sourceforge.net>
send remarks or bug reports to gerbier\@users.sourceforge.net

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

";

	$text->insert( 'end', $baratin );
	$text->see('1.0');
}
#####################################################################
# a global var for do_save_log and do_load_log
my $log_types =
  [ [ 'log files', '.log' ], [ 'text files', '.txt' ], [ 'All Files', '*', ], ];
#####################################################################
# save change screen to a log file
sub do_save_log($) {
	my $main = shift;

	my @logs = $change_text->get( '1.0', 'end' );
	my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday ) = gmtime(time);
	$year += 1900;
	$mon  += 1;
	my $date    = $year . "_" . "$mon" . "_" . $mday . "_" . $hour . "_" . $min;
	my $logname = "afick." . $date . ".log";
	my $filename = $main->getSaveFile(
		-initialfile => "$logname",
		-filetypes   => $log_types
	);

	if ($filename) {
		print "log in $filename\n";
		if ( !open( LOG, ">", $filename ) ) {
			warning("can not write on $filename");
		}
		else {
			print LOG @logs;
			close LOG;
		}
	}
}
#####################################################################
# load a log file into change part
sub do_load_log($) {
	my $main = shift;

	my @logs = $change_text->get( '1.0', 'end' );
	my $filename = $main->getOpenFile( -filetypes => $log_types );
	if ($filename) {
		clear_text($change_text);
		clear_text($warning_text);

		if ( !open( LOG, "<", $filename ) ) {
			warning("can not read $filename");
		}
		else {
			while ( my $line = <LOG> ) {
				$change_text->insert( 'end', $line );
			}
			$change_text->see('1.0');
			close LOG;
		}
	}
}
#####################################################################
# a global var for select_config and save_config
my $conf_types = [ [ 'config files', '.conf' ], [ 'All Files', '*', ], ];
#####################################################################
# to select a config file
sub select_config($$) {
	my $FenetreP = shift;
	my $entree   = shift;

	my $filename = $FenetreP->getOpenFile( -filetypes => $conf_types );
	if ( defined $filename and $filename ne '' ) {
		$entree->delete( 0, 'end' );
		$entree->insert( 0, $filename );
		$entree->xview('end');
	}
}
#####################################################################
# to save a config file
sub save_config($$$) {
	my $main     = shift;
	my $text     = shift;
	my $top      = shift;
	my $initfile = $configfile;
	$initfile =~ s?/?\\?g;
	my $filename = $main->getSaveFile(
		-initialfile => "$initfile",
		-filetypes   => $conf_types
	);

	my @conf = $text->get( '1.0', 'end' );
	if ( !open( CONF, ">", $configfile ) ) {
		warning("can not write in $configfile : $!");
	}
	else {
		print CONF @conf;
		close CONF;
	}
	$top->destroy();
}
#####################################################################
# open selected config file
sub open_config($) {
	my $main = shift;

	$configfile = $configfile || $default_conf;

	my $top = $main->Toplevel;
	$top->Label( -text => $configfile )->pack();
	my $text = $top->Scrolled(
		'Text',
		-scrollbars => 'e',
		-height     => 25,
		-width      => 128,
		-wrap       => 'word'
	)->pack( -side => "left", -expand => 1, -fill => 'both' );

#$top->Button( -text => "save" ,  -command => sub { save_config($main, $text, $top) } )->pack();
	my $SaveButton = $top->Button(
		-text    => "save",
		-command => sub { save_config( $main, $text, $top ) }
	);
	$SaveButton->pack();
	$top->Button( -text => "quit", -command => [ $top => 'destroy' ] )
	  ->pack( -after => $SaveButton );

	if ( !open( CONF, "<", $configfile ) ) {
		warning("can not read $configfile : $!");
		return;
	}
	else {
		while (<CONF>) {
			$text->insert( 'end', $_ );
		}
		close(CONF);
		$text->see('1.0');
	}

}
#####################################################################
sub usage($) {

	my $version = shift (@_);
	print <<EOHELP
afick-tk ($version) : a graphical interface to afick
Usage: afick-tk [options]
Options:
--config_file|-c config
	name of config file to use
--full_newdel|-f
	report full  information on new and deleted directories
--help|-h
	this help
--missing_files|-m
	warn about files declared in config files which does not exists
--running_files|-r
	warn about files modified during program run
--dead_symlinks|-s
	warn about dead symlinks
--timing|-t
	Print timing statistics
--verbose|-v
	verbose mode (for debugging purpose)
--version|-V
	print program version

#################################################################
This script is intended to provide a means for
detecting changes made to files, via a regular
comparison of MD5 hashes to an established baseline. 
#################################################################
Copyright (c) 2002 Eric Gerbier <gerbier\@users.sourceforge.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
#################################################################
EOHELP
}
#####################################################################
##                       main
#####################################################################

# options
Getopt::Long::Configure('no_ignore_case');
unless (
	GetOptions(
		"config_file|c=s"  => \$configfile,
		"full_newdel|f!"   => \$Report_full_newdel,
		"help|h"           => \$help,
		"missing_files|m!" => \$Warn_missing_file,
		"running_files|r!" => \$Running,
		"dead_symlinks|s!" => \$Warn_dead_symlinks,
		"timing|t!"        => \$Timing,
		"verbose|v!"       => \$verbose,
		"version|V"        => \$Version
	)
  )
{
	usage($version);

	# can not use warning : no widget this time
	die ("incorrect option !");
}

if ($help) {
	usage($version);
	exit;
}

if ($Version) {
	print "afick-tk  version $version\n";
	exit;
}

# tk page
my $main = MainWindow->new( -title => "afick-tk $version" );

my $side = "top";

# we will have
# frame 1 : action
# frame 2 : outpout
# frame 3 : warning
# frame 4 : progress bar
# frame 5 : configuration
# frame 6 : options

# frame 1 : actions
####################
my $frame1 = $main->Frame();
$frame1->pack( -side => $side );

$frame1->Label( -text => "actions" )->pack( -side => "left" );
$frame1->Button( -text => "init", -command => sub { do_action("-i") } )
  ->pack( -side => "left" );
$frame1->Button( -text => "update", -command => sub { do_action("-u") } )
  ->pack( -side => "left" );
$frame1->Button( -text => "compare", -command => sub { do_action("-k") } )
  ->pack( -side => "left" );
$frame1->Button( -text => "print", -command => sub { do_action("-p") } )
  ->pack( -side => "left" );
$frame1->Button( -text => "save", -command => sub { do_save_log($main) } )
  ->pack( -side => "left" );
$frame1->Button( -text => "load", -command => sub { do_load_log($main) } )
  ->pack( -side => "left" );
$frame1->Button( -text => "help", -command => sub { do_help($main) } )
  ->pack( -side => "left" );
$frame1->Button( -text => "about", -command => sub { do_about($main) } )
  ->pack( -side => "left" );
$frame1->Button( -text => "exit", -command => sub { exit } )
  ->pack( -side => "left" );

# frame 2 : text vue
####################
my $frame2 = $main->LabFrame( -label => "changes section" );
$frame2->pack( -side => $side, -expand => 1, -fill => 'x' );

$change_text = $frame2->Scrolled(
	'ROText',
	-scrollbars => 'e',
	-height     => 25,
	-width      => 128,
	-wrap       => 'word'
)->pack( -side => $side, -expand => 1, -fill => 'both' );

my $adj1 = $main->Adjuster();
$adj1->packAfter( $frame2, -side => $side );

# frame 3 : warning vue
####################
my $frame3 = $main->LabFrame( -label => "warnings section" );
$frame3->pack( -side => $side, -expand => 1, -fill => 'x' );

$warning_text = $frame3->Scrolled(
	'ROText',
	-scrollbars => 'e',
	-height     => 5,
	-width      => 128,
	-wrap       => 'word'
)->pack( -side => $side, -expand => 1, -fill => 'both' );

# frame 4 : progress
####################
my $frame4 = $main->LabFrame( -label => "progress section" );
$frame4->pack( -side => $side, -expand => 1, -fill => 'x' );

$frame4->Checkbutton(
	-text     => "display progress",
	-variable => \$ToggleProgress
)->pack( -side => $side );

$progress_text = $frame4->ROText(
	-height => 1,
	-width  => 128,
	-wrap   => 'word'
)->pack( -side => $side );

$progress_bar = $frame4->ProgressBar(
	-length      => 780,
	-colors      => [ 0, 'green' ],
	-troughcolor => 'grey55',
	-variable    => \$progress_percent
)->pack( -side => $side );

# frame 5  : configuration file
####################
my $frame5 = $main->LabFrame( -label => "configuration section" );
$frame5->pack( -side => $side );

$frame5->Label( -text => "configuration file" )->pack( -side => "left" );
my $entry =
  $frame5->Entry( -textvariable => \$configfile, -width => 30 )
  ->pack( -side => "left", -expand => 'yes' );
$frame5->Button(
	-text    => 'Select...',
	-command => sub { select_config( $main, $entry ) }
)->pack( -side => "left" );
$frame5->Button(
	-text    => 'View/Edit',
	-command => sub { open_config($main) }
)->pack( -side => "left" );

# frame 6 : options
###################
my $frame6 = $main->Frame();

#$frame6->pack( -side => $side );
$frame6->pack( -side => "bottom" );

$frame6->Label( -text => "options" )->pack( -side => "left" );

$frame6->Optionmenu(
	-options  => [@verbose_values],
	-variable => \$verbose
)->pack( -side => "left" );
$frame6->Optionmenu(
	-options  => [@missing_values],
	-variable => \$Warn_missing_file
)->pack( -side => "left" );
$frame6->Optionmenu(
	-options  => [@dead_sym_values],
	-variable => \$Warn_dead_symlinks
)->pack( -side => "left" );
$frame6->Optionmenu(
	-options  => [@full_newdel_values],
	-variable => \$Report_full_newdel
)->pack( -side => "left" );
$frame6->Optionmenu(
	-options  => [@running_files_values],
	-variable => \$Running
)->pack( -side => "left" );
$frame6->Optionmenu(
	-options  => [@timing_values],
	-variable => \$Timing
)->pack( -side => "left" );

MainLoop;

__END__

=head1 NAME

afick-tk : a graphical interface for afick (Another File Integrity Checker)

=head1 SYNOPSIS

=head1 OPTIONS

=head1 DESCRIPTION

=cut
